# vim: expandtab tabstop=4 shiftwidth=4
'''
Find indices of peaks
'''

from typing import Any
from numpy import array as nparray
from numpy import append, arange, argmax, copy, inf, ones, pad

def find_peaks(inarray: nparray, min_height: float=1, min_dist: int=20, num_peaks: Any=inf) -> nparray:
    '''
    Find indices of peak values in order of greatest value to least

    Parameters
    —————
    inarray:
        numpy.array containing real samples
    min_height: float
        threshold a value must exceed to be considered a peak
        Default: 1
    min_dist: int
        Minimum distance (in samples) allowed between peaks
        Default: 20
    num_peaks: int or numpy.inf
        Maximum number of peak indices to return
        Default: numpy.inf (Return all peaks)

    Returns at most num_peaks indices in a numpy array.
    '''

    # Output will be integer list of indices
    out = nparray([], dtype=int)

    # In case num_peaks <= 0 output emptry array
    if num_peaks <= 0:
        return out

    # Flatten array
    arr = copy(inarray).flatten()

    # Pad with 'min_dist' elements to both ends of 'arr'
    # Prevents possible error when overwriting array with "zero-ed" elements
    arr = pad(arr, (min_dist, min_dist), "constant", constant_values=(min_height-1, min_height-1))

    # Find index with maximum value
    midx = argmax(arr)
    while arr[midx] > min_height:
        # Append maximal index to output array
        out = append(out, midx-min_dist)
        # Check if we reached the maximum number of peaks
        if len(out) >= num_peaks:
            return out

        # "Zero" array values withing min_dist of maximal index
        arr[arange(midx-min_dist, midx+min_dist+1)] = ones(2*min_dist+1)*(min_height-1)

        # Find index with maximum value for next iteration
        midx = argmax(arr)

    return out


def findpeaks(*args, **kwargs):
    '''
	Alias for find_peaks.
	'''
    return find_peaks(*args, **kwargs)



# vim: expandtab tabstop=4 shiftwidth=4

''' Cross-Correlation with normalization
'''

from scipy.signal import convolve, correlate

import numpy as np

def signal_correlation(in1: np.array, in2: np.array, norm=True, mode='full', method='auto') -> np.array:
    '''
    Cross-correlates two arrays and normalizes the output

    Parameters
    ----------
    in1:
        numpy.array containing first signal
    in2:
        numpy.array containing second signal
    norm: bool
        Flag to normalize the output.
        Default is True
    mode: string
        Size of output. Options 'full', 'same', 'valid'
        Default is 'full'
        full:   Entire cross-correlation of inputs
        same:   Output is the same size at in1
        valid:  Only output values that do not depend on zero padding
    method: string
        Calculation method. Options 'auto', 'direct', 'fft'
        Default is 'auto'
        auto:   Automatically determine the best method
        direct: Use correlation definition
        fft:    Use Fast Fourier Transform for possibly faster computation

    Returns the correlation vector as a numpy array.
    Warning: Normalization does not check for division by 0 errors.
    '''

    arr_l = np.array(in1)
    arr_s = np.array(in2)
    # If norm is set to False, unnormalized output will be returned
    out_arr = correlate(arr_l, arr_s, mode=mode, method=method)
    # Default output is set to normalized
    if norm:
        s_pow = np.sqrt(np.sum(arr_s*arr_s.conj()))
        l_pow = np.sqrt(convolve(arr_l*arr_l.conj(), np.ones(len(arr_s)), mode=mode, method=method))*s_pow
        out_arr = out_arr/l_pow
    return out_arr

def sigcorr(*args, **kwargs):
    '''
    Alias for signal_correlation.
    '''
    return signal_correlation(*args, **kwargs)

def signal_correlate(*args, **kwargs):
    '''
    Alias for signal_correlation.
    '''
    return signal_correlation(*args, **kwargs)
